//*****************************************************************************
// File Name            : read.v
//-----------------------------------------------------------------------------
// Function             : read_mem
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
//Rg
// History
// -.-- 2012/7/27
// -.-- 2012/8/17
//*****************************************************************************
module read_mem (
  clk,
  ram_clk,
  reset_n, 
  rd_start, 
  rd_data,
  rd_data_en,
  u_rreq,   
  u_rack,   
  u_radr,     
  u_rd_da_en,    
  u_rd_da,
  i_page,
  mem_read_flg
);

  input clk;    //output clk
  input ram_clk;
  input reset_n;
  input rd_start;          //vram read req
  output [7:0]rd_data;  //vram read data
  output rd_data_en;  //vram read data enable
 //
  output u_rreq;        //user read req
  input u_rack;       //user read ack
  output [19:0]u_radr; //user read address
  input u_rd_da_en;     //user read  enable
  input [15:0]u_rd_da; //user read  data
  input      i_page ; //camera input page 
  output     mem_read_flg; //mem read flg
   

  parameter p_line_end = 8'd319; //line buf end

  parameter p_r_idle   = 4'h0;
  parameter p_r_req    = 4'h1;
  parameter p_r_rd     = 4'h2;
  parameter p_r_send   = 4'h3;
  parameter p_r_rd_end = 4'h4;
   
  //parameter p_base_end = 13'd6999; //usrt 115.2KHz
  //parameter p_base_end = 13'd9999; //usrt 921.6KHz
  parameter p_base_end = 13'd999; //usrt 921.6KHz
 
  reg [7:0]rd_data;  //vram read data
  reg rd_data_en;  //vram read data enable

  reg  line_buf_out_we;
  reg  [1:0] line_buf_out_wc;
  reg  [9:0] line_buf_out_wadr;
  reg  [9:0] line_buf_out_wadr_tmp;
  reg  [23:0]line_buf_out_wda;
   
  reg rd_start_d1;
  reg rd_start_d2;
  reg rd_start_rc_d1;
  reg rd_start_rc_d2;
  reg rd_start_rc_d3;
  wire  rd_send_sig;
  reg [3:0]u_r_state;        //user write req
  reg      send_state;
  reg [9:0] u_r_cnt;
  reg [8:0] u_r_vadr;
  reg       u_rreq;
  reg       u_rreq_d1;
  reg [19:0]u_radr; //user read address
  reg         u_rd_da_en_d1;
  reg  [15:0] u_rd_da_d1;
  reg  [2:0] u_rd_da_cnt;
  reg  [2:0] u_rd_wadr;
  reg  send_end_d1;
  reg  send_end_d2;
  reg  send_end_d3;
  wire send_end_sig;
  reg  read_end_d1;
  reg  read_end_d2;
 
  //clk 
  reg   send_state_d1;
  reg   send_state_d2;
  reg   send_state_d3;
  reg  send_end;
  reg mem_read_flg; //mem read flg
  wire  rd_start_sig;
  wire [15:0] buf_b_q;
  reg  [12:0]  rd_cnt_base;
  reg  rd_cnt_base_en;
  reg  [3:0]  rd_cnt;
  reg  [5:0]  rd_blk_cnt;
  reg  [8:0]  rd_line_cnt;
  reg         rd_cnt_0_d1;
  reg         buf_b_q_sel;
  reg         read_end;
   
//-----------------------------------------------------------------------------      
//ram_clk    
//line buf out write
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    begin
      rd_start_rc_d1<= 1'b0;
      rd_start_rc_d2<= 1'b0;
      rd_start_rc_d3<= 1'b0;
    end
  else
    begin
      rd_start_rc_d1<= rd_start_d2;
      rd_start_rc_d2<= rd_start_rc_d1;
      rd_start_rc_d3<= rd_start_rc_d2;
    end

always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rd_da_en_d1 <= 1'b0;
  else
    u_rd_da_en_d1 <= u_rd_da_en;
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rd_da_d1 <= 16'b0;
  else
    if (u_rd_da_en==1'b1)
      u_rd_da_d1 <= u_rd_da;
    else
      u_rd_da_d1 <= u_rd_da_d1;
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rd_da_cnt <= 3'd0;
  else
    if (u_rd_da_en==1'b1)
      u_rd_da_cnt <= u_rd_da_cnt + 3'd1;
    else
      u_rd_da_cnt <= 3'b0;
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rd_wadr <= 3'd0;
  else
    u_rd_wadr <= u_rd_da_cnt;
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_r_state <= p_r_idle;
  else
    case(u_r_state)
      //-------------------------------------
      p_r_idle:begin
        if ((rd_start_rc_d3==1'b0)&&( rd_start_rc_d2==1'b1))
          u_r_state <= p_r_req;
        else
          u_r_state <= p_r_idle;
      end
      //-------------------------------------
      p_r_req:begin
        if (u_rack==1'b1)
          u_r_state <= p_r_rd;
        else
          u_r_state <= p_r_req;
      end
      //-------------------------------------
      p_r_rd:begin
        if ((u_rd_da_en_d1==1'b1)&&(u_rd_da_en==1'b0))
          u_r_state <= p_r_send;
        else
          u_r_state <= p_r_rd;
      end
      //-------------------------------------
      p_r_send:begin
        if(send_end_sig==1'b1)
          u_r_state <= p_r_rd_end;
        else
          u_r_state <=  p_r_send;
      end
      //-------------------------------------
      p_r_rd_end:begin
        if  (read_end_d2==1'b1)
        u_r_state <= p_r_idle;
        else
          u_r_state <=p_r_req ;
      end
    endcase // case(u_r_state)
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    send_state <= 1'b0;
  else
    if (u_r_state==p_r_send)
      send_state <= 1'b1;
    else
      send_state <= 1'b0;
     
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_r_cnt <= 10'd0;
  else
    if (u_r_state == p_r_idle)
      u_r_cnt <= 10'b0;
    else if (u_rd_da_en==1'b1)
      if (u_r_cnt==10'd479)
        u_r_cnt <= 10'd0;
      else
        u_r_cnt <= u_r_cnt + 1'b1;
    else
      u_r_cnt <= u_r_cnt;
   

always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_r_vadr <= 9'h000;
  else
    if (u_r_state==p_r_req)
      u_r_vadr <= {rd_line_cnt};
    else
      u_r_vadr <= u_r_vadr;


always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rreq <= 1'b0;
  else
    if (u_r_state == p_r_req)
      u_rreq <= 1'b1;
    else if (u_r_state == p_r_rd)
      u_rreq <= 1'b0;
    else 
      u_rreq <= u_rreq;

always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_rreq_d1 <= 1'b0;
  else
    u_rreq_d1 <=u_rreq ;
   
always @ (posedge ram_clk or negedge reset_n )
  if (reset_n==1'b0)
    u_radr <= 20'h00000;
  else
    if(( u_rreq==1'b1)&&( u_rreq_d1==1'b0)) 
      u_radr <= {~i_page,u_r_vadr,u_r_cnt}; //J݃y[WƂ͕ʂ̖ʂ[h
    else
      u_radr <= u_radr;
  

always @ (posedge ram_clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    send_end_d1 <= 1'b0;
    send_end_d2 <= 1'b0; 
    send_end_d3 <= 1'b0; 
  end
else
  begin
    send_end_d1 <= send_end;
    send_end_d2 <= send_end_d1; 
    send_end_d3 <= send_end_d2; 
  end
   
assign send_end_sig = ((send_end_d3==1'b0)&&(send_end_d2==1'b1))?1'b1:1'b0;
   
always @ (posedge ram_clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    read_end_d1 <= 1'b0;
    read_end_d2 <= 1'b0; 
  end
else
  begin
    read_end_d1 <= read_end;
    read_end_d2 <= read_end_d1; 
  end
   
  
//------------------------------------------------------------------------------------  
//clk
//line buf out read

always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    begin
       rd_start_d1 <= 1'b0;
       rd_start_d2 <= 1'b0;
    end
  else
    begin
       rd_start_d1 <= rd_start;
       rd_start_d2 <= rd_start_d1;
    end
       
 assign  rd_start_sig = (( rd_start_d1==1'b1)&&( rd_start_d2==1'b0))?1'b1:1'b0;
   
   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    send_state_d1 <= 1'b0;
    send_state_d2 <= 1'b0; 
    send_state_d3 <= 1'b0; 
  end
else
  begin
    send_state_d1 <= send_state;
    send_state_d2 <= send_state_d1; 
    send_state_d3 <= send_state_d2; 
  end

assign rd_send_sig = ((send_state_d2==1'b1)&&(send_state_d3==1'b0))?1'b1:1'b0;


always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  mem_read_flg <= 1'b0;
else
  if (rd_start==1'b1)
    mem_read_flg <= 1'b1 ;
  else
    if (( send_end==1'b1)&&(rd_blk_cnt==6'd59)&&(rd_line_cnt==9'd479)) 
      mem_read_flg <= 1'b0;
    else
      mem_read_flg <= mem_read_flg;
   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  read_end <= 1'b0;
else
  if (((rd_cnt==4'd15)&&(rd_cnt_base == p_base_end))&&(rd_blk_cnt==6'd59)&&(rd_line_cnt==9'd479)) 
    read_end <= 1'b1;
  else if (rd_start==1'b1)
    read_end <= 1'b0;
  else
    read_end <= read_end;
   
always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    rd_cnt_base_en <= 1'b0;
  else
    if (rd_send_sig==1'b1)
      rd_cnt_base_en <= 1'b1;
    else if (send_end==1'b1)
      rd_cnt_base_en <= 1'b0;
    else
      rd_cnt_base_en <= rd_cnt_base_en ;
   
always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    rd_cnt_base <= 13'd0; 
  else
    if (rd_cnt_base_en ==1'b1)
      if (rd_cnt_base == p_base_end)
        rd_cnt_base <= 13'd0;
      else 
        rd_cnt_base <=  rd_cnt_base + 13'd1;
    else
      rd_cnt_base <= 13'd0;

always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    rd_cnt <= 4'd0; 
  else
    if (rd_cnt_base_en ==1'b1)
      if (rd_cnt_base == p_base_end)
        rd_cnt <= rd_cnt + 4'd1;
      else
        rd_cnt <= rd_cnt ;
    else 
      rd_cnt <= 4'd0;

always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    send_end <= 1'b0;
  else
    if ((rd_cnt==4'd15)&&(rd_cnt_base == p_base_end))
      send_end <= 1'b1;
    else      
      send_end <= 1'b0;

always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    rd_blk_cnt <= 6'd0; 
  else
    if(rd_start_sig==1'b1)
      rd_blk_cnt <= 6'd0; 
    else 
      if (rd_cnt_base_en ==1'b1)
        if ( send_end==1'b1)
          if (rd_blk_cnt==6'd59)
            rd_blk_cnt <= 6'd0; 
          else
            rd_blk_cnt <= rd_blk_cnt + 8'd1;
        else
          rd_blk_cnt <= rd_blk_cnt;
      else 
          rd_blk_cnt <= rd_blk_cnt;

always @ (posedge clk or negedge reset_n )
  if (reset_n==1'b0)
    rd_line_cnt <= 9'd0; 
  else
    if(rd_start_sig==1'b1)
      rd_line_cnt <= 9'd0; 
    else 
      if (rd_cnt_base_en ==1'b1)
        if (( send_end==1'b1)&&(rd_blk_cnt==6'd59))
          rd_line_cnt <= rd_line_cnt + 9'd1;
        else
          rd_line_cnt <= rd_line_cnt;
      else 
          rd_line_cnt <= rd_line_cnt;

   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  rd_cnt_0_d1 <= 1'b0;
else
  rd_cnt_0_d1 <= rd_cnt[0];

always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  buf_b_q_sel <= 1'b0;             
else
  if( rd_cnt_0_d1==1'b0)
    buf_b_q_sel <= 1'b0;
  else
    buf_b_q_sel <= 1'b1;

always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  rd_data <= 8'h00;
else
  if (buf_b_q_sel==1'b0) 
    rd_data <= buf_b_q[15:8];
  else
    rd_data <=buf_b_q[7:0];

always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  rd_data_en <= 1'b0;
else
  if ((rd_cnt_base_en ==1'b1)&&(rd_cnt_base == p_base_end))
    rd_data_en <= 1'b1;
  else
    rd_data_en <= 1'b0;    

ram_16b_8w read_mem_buf(
        .data(     u_rd_da_d1) ,
        .wrclock(   ram_clk),
        .rdclock(  clk),
        .rdaddress( rd_cnt[3:1]),
        .wraddress( u_rd_wadr ),
        .wren(      u_rd_da_en_d1),
        .q(         buf_b_q  ));
     
endmodule








